module AlexaToolbox
  # Echo can send 3 types of requests
  # - LaunchRequest: The app was launched using phrasing such as 'Alexa open ______'.
  # - IntentRequest: An intent was requested.
  # - SessionEndedRequest: Session was closed either by user or timeout.
  
  class Request
    require 'json'
    require 'alexa_toolbox/session'
    require 'alexa_toolbox/intent'
    require 'alexa_toolbox/audioplayer'
    require 'alexa_toolbox/context'
    attr_reader :version, :session, :json, :options, :request_id, :locale, :type, :intent, :audioplayer, :error, :cause, :context

    def self.default_options
      @default_options ||= {
        :application_id => ''
      }
    end

    def initialize(json_request, options = {})
      @options = options if options.key?(:application_id)

      @request_id = json_request[:request][:requestId]
      raise ArgumentError, 'Request ID not present' if @request_id.nil?
      @version = json_request.key?(:version) ? json_request[:version] : ""
      @locale = json_request.key?(:request) && json_request[:request].key?(:locale) ? json_request[:request][:locale] : ""
      @error = json_request.key?(:request) && json_request[:request].key?(:error) ? json_request[:request][:error] : ""
      @cause = json_request.key?(:request) && json_request[:request].key?(:cause) ? json_request[:request][:cause] : ""
      @json   = json_request
      @type = json_request[:request][:type][0..10] == "AudioPlayer" ? "AudioPlayer" : json_request[:request][:type]

      @intent = @type == "IntentRequest" ? AlexaToolbox::Intent.new(json_request[:request][:intent]) : nil
      @audioplayer = @type == "AudioPlayer" ? AlexaToolbox::AudioPlayer.new(json_request[:request]) : nil
      @session = json_request.key?(:session) ? AlexaToolbox::Session.new(json_request[:session]) : nil
      @context = json_request.key?(:context) ? AlexaToolbox::Context.new(json_request[:context]) : nil
    end

    def valid?
      self.options[:application_id] && ((!self.session.nil? && self.options[:application_id] == self.session.application_id) || (!self.context.nil? && self.options[:application_id] == self.context.application_id))
    end

    def user_terminated?
      if self.type != "SessionEndedRequest"
        return nil
      end
      return self.json[:request][:reason] == "USER_INITIATED"
    end
  end

  # Builds a new request for Alexa, primarily for testing purposes
  def self.build_request(json_request, application_id = "")
    json_request = self.transform_keys_to_symbols(json_request)
    raise ArgumentError, 'Invalid Alexa Request. Missing session, request, version, or application id.' unless AlexaToolbox.valid_alexa?(json_request)
    Request.new(json_request,{ :application_id => application_id })
  end
end
